//Copyright (c) 1999, Art Gittleman
//This example is provided WITHOUT ANY WARRANTY either expressed or implied.

/* Uses wait and notify to enable
 * producer and consumer threads to
 * cooperate in using a buffer.
 */
  
public class PutGet {
  public static final int size = 3;
  class Buffer {
    int[] buffer = new int [size];             
    int putpos=0;
    int getpos=0;
    int number=0;

    public synchronized void put(int value) throws InterruptedException {
      if (number==size) {
        System.out.println("Cannot put -- Buffer full");
        wait();
      }
      number++;
      buffer[putpos] = value;
      System.out.println("Put "+value);
      putpos=(putpos+1)%size;
      if (number == 1) notify();         
    }
    public synchronized int get() throws InterruptedException {
      if (number==0) {
         System.out.println("Buffer empty");
         wait();
      }
      number--;
      int n = buffer[getpos];
      System.out.println("Get "+n);
      getpos=(getpos+1)%size;
      if (number==size-1) notify();    
      return n;
    }
  }

  class Producer extends Thread {
     Buffer buf;
     int time;

     Producer(Buffer b, int t) { 
        buf = b;
        time = t;
     }			

     public void run() {
       for(int i=1; i<=10; i++)
         try {
             buf.put(i);
             sleep(time);
           }catch (InterruptedException e){
              e.printStackTrace();
           }
     }
  }

  class Consumer extends Thread {
     Buffer buf;
     int time;

     Consumer(Buffer b, int t) { 
       buf = b;
       time = t;
     }

     public void run() {
       for (int i=1; i<=10;i++)
         try {
           buf.get();
           sleep(time);
         }catch (InterruptedException e) {
               e.printStackTrace();
         }
     }
  }

  public static void main(String[] args) {
    PutGet pg = new PutGet();
    Buffer b = pg.new Buffer();
    if (args.length!=2) {
	System.out.println("Usage: PutGet  <Producer TO>  <Consumers TO>");
        System.exit(0);
    }
    Producer p = pg.new Producer(b,Integer.parseInt(args[0]));
    Consumer c = pg.new Consumer(b,Integer.parseInt(args[1]));
    p.start();
    c.start();
  }
}

